package com.appier.appier_flutter

import android.app.Activity
import android.content.Context
import androidx.annotation.NonNull

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import com.quantumgraph.sdk.QG
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import org.json.JSONObject

/** AppierFlutterPlugin */
class AppierFlutterPlugin: FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel

  private lateinit var context: Context
  private lateinit var activity: Activity

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "appier_flutter")
    context = flutterPluginBinding.applicationContext
    channel.setMethodCallHandler(this)
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    when (call.method) {
        "configure" -> {
          val appId = call.argument<String>("appId")
          var senderId: String = ""
          if (call.hasArgument("senderId")) {
            senderId = call.argument<String>("senderId")!!
          }
          if (senderId.isNotEmpty())
            QG.initializeSdk(activity.application, appId, senderId)
          else
            QG.initializeSdk(activity.application, appId)

          result.success("SDK initialised")
        }
        "logEvent" -> {
          val eventName = call.argument<String>("eventName")!!
          var parameters = JSONObject()
          if (call.hasArgument("parameters")) {
            parameters = call.argument<JSONObject>("parameters")!!
          }
          var vts = 0.0
          if (call.hasArgument("vts")) {
            vts = call.argument<Double>("vts")!!
          }
          var vtsCurr = ""
          if (call.hasArgument("vts")) {
            vtsCurr = call.argument<String>("vtsCurr")!!
          }
          QG.getInstance(context).logEvent(eventName, parameters, vts, vtsCurr)
        }
        else -> {
          result.notImplemented()
        }
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }

  override fun onDetachedFromActivity() {
    TODO("Not yet implemented")
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    TODO("Not yet implemented")
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    activity = binding.activity;
  }

  override fun onDetachedFromActivityForConfigChanges() {
    TODO("Not yet implemented")
  }
}
